@@ -2,7 +2,7 @@
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
  * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
- * Copyright (c) 1999-2024 The strace developers.
+ * Copyright (c) 1999-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -611,7 +611,7 @@ extern kernel_ulong_t get_rt_sigframe_addr(struct tcb *);
  * @return       String literal corresponding to the syscall number in case
  *               the latter is valid;  NULL otherwise.
  */
-extern const char *syscall_name_arch(kernel_ulong_t nr, unsigned int arch,
+extern const char *syscall_name_arch(unsigned long long nr, unsigned int arch,
 				     const char **prefix);
 /**
  * Convert a syscall name to the corresponding (shuffled) syscall number.
@@ -1440,6 +1440,7 @@ extern void qualify_write(const char *);
 extern void qualify_fault(const char *);
 extern void qualify_inject(const char *);
 extern void qualify_kvm(const char *);
+extern void qualify_namespace(const char *);
 extern unsigned int qual_flags(const unsigned int);
 
 # define DECL_IOCTL(name)						\
@@ -1449,6 +1450,7 @@ name ## _ioctl(struct tcb *, unsigned int request, kernel_ulong_t arg)	\
 
 DECL_IOCTL(counter);
 DECL_IOCTL(dm);
+DECL_IOCTL(epoll);
 DECL_IOCTL(evdev);
 DECL_IOCTL(fs_0x94);
 DECL_IOCTL(fs_f);
@@ -1595,6 +1597,8 @@ extern void kvm_run_structure_decoder_init(void);
 extern void kvm_vcpu_info_free(struct tcb *);
 # endif
 
+extern void namespace_auxstr_init(void);
+
 extern void maybe_load_task_comm(struct tcb *tcp);
 /* Print the contents of /proc/$pid/comm. */
 extern void print_pid_comm(int pid);
