@@ -9,12 +9,23 @@
 
 SYS_FUNC(fchownat)
 {
+	/* dirfd */
 	print_dirfd(tcp, tcp->u_arg[0]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* pathname */
 	printpath(tcp, tcp->u_arg[1]);
-	printuid(", ", tcp->u_arg[2]);
-	printuid(", ", tcp->u_arg[3]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* owner */
+	printuid(tcp->u_arg[2]);
+	tprint_arg_next();
+
+	/* group */
+	printuid(tcp->u_arg[3]);
+	tprint_arg_next();
+
+	/* flags */
 	printflags(at_flags, tcp->u_arg[4], "AT_???");
 
 	return RVAL_DECODED;
