@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2016-2023 The strace developers.
+ * Copyright (c) 2016-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -105,6 +105,7 @@ decode_fd_str_to_uint(const char *str)
 		{ DECODE_FD_PATH,      "path" },
 		{ DECODE_FD_SOCKET,    "socket" },
 		{ DECODE_FD_DEV,       "dev" },
+		{ DECODE_FD_EVENTFD,   "eventfd" },
 		{ DECODE_FD_PIDFD,     "pidfd" },
 		{ DECODE_FD_SIGNALFD,  "signalfd" },
 	};
@@ -632,6 +633,16 @@ qualify_kvm(const char *const str)
 	}
 }
 
+void
+qualify_namespace(const char *const str)
+{
+	if (strcmp(str, "new") == 0) {
+		namespace_auxstr_init();
+	} else {
+		error_msg_and_die("invalid -e namespace= argument: '%s'", str);
+	}
+}
+
 #ifdef ENABLE_SECONTEXT
 struct number_set *secontext_set;
 
@@ -698,6 +709,7 @@ static const struct qual_options {
 	{ "decode-pid",	qualify_decode_pid },
 	{ "decode-pids", qualify_decode_pid },
 	{ "secontext",  qualify_secontext },
+	{ "namespace",  qualify_namespace },
 };
 
 void
