@@ -5,7 +5,7 @@
  * Copyright (c) 1996-1999 Wichert Akkerman <wichert@cistron.nl>
  * Copyright (c) 2000 PocketPenguins Inc.  Linux for Hitachi SuperH
  *                    port by Greg Banks <gbanks@pocketpenguins.com>
- * Copyright (c) 1999-2023 The strace developers.
+ * Copyright (c) 1999-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -279,8 +279,8 @@ SYS_FUNC(mremap)
 	/* flags */
 	printflags64(mremap_flags, tcp->u_arg[3], "MREMAP_???");
 
-	if ((tcp->u_arg[3] & (MREMAP_MAYMOVE | MREMAP_FIXED)) ==
-	    (MREMAP_MAYMOVE | MREMAP_FIXED)) {
+	if (tcp->u_arg[3] & (MREMAP_FIXED | MREMAP_DONTUNMAP) &&
+	    tcp->u_arg[3] & MREMAP_MAYMOVE) {
 		/* new_address */
 		tprint_arg_next();
 		printaddr(tcp->u_arg[4]);
@@ -492,6 +492,22 @@ SYS_FUNC(remap_file_pages)
 	return RVAL_DECODED;
 }
 
+SYS_FUNC(mseal)
+{
+	/* addr */
+	printaddr(tcp->u_arg[0]);
+	tprint_arg_next();
+
+	/* length */
+	PRINT_VAL_U(tcp->u_arg[1]);
+	tprint_arg_next();
+
+	/* flags, reserved for future use */
+	PRINT_VAL_X(tcp->u_arg[2]);
+
+	return RVAL_DECODED;
+}
+
 #if defined(POWERPC)
 static bool
 print_protmap_entry(struct tcb *tcp, void *elem_buf, size_t elem_size, void *data)
