@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2016-2017 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2017-2023 The strace developers.
+ * Copyright (c) 2017-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -713,7 +713,9 @@ tprint_sysret_end(void)
 	do {								\
 		tprints_field_name(#field_);				\
 		const char *nr_prefix_ = NULL;				\
-		const char *name = syscall_name_arch((where_).field_,	\
+		const unsigned long long ull_nr =			\
+			zero_extend_signed_to_ull((where_).field_);	\
+		const char *name = syscall_name_arch(ull_nr,		\
 					(audit_arch_), &nr_prefix_);	\
 		if (xlat_verbose(xlat_verbosity) != XLAT_STYLE_ABBREV	\
 		    || !nr_prefix_)					\
