@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2021 Dmitry V. Levin <ldv@strace.io>
+ * Copyright (c) 2018-2025 Dmitry V. Levin <ldv@strace.io>
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -11,6 +11,7 @@
 extern bool ptrace_get_syscall_info_supported;
 extern bool test_ptrace_get_syscall_info(void);
 extern void print_ptrace_syscall_info(struct tcb *, kernel_ulong_t addr,
-				      kernel_ulong_t len);
+				      kernel_ulong_t user_len,
+				      kernel_ulong_t kernel_len);
 
 #endif /* !STRACE_PTRACE_SYSCALL_INFO_H */
