@@ -3,7 +3,7 @@
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
  * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * Copyright (c) 1996-1999 Wichert Akkerman <wichert@cistron.nl>
- * Copyright (c) 1999-2021 The strace developers.
+ * Copyright (c) 1999-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -36,7 +36,7 @@ print_rlim64_t(uint64_t lim) {
 		return;
 
 	(xlat_verbose(xlat_verbosity) == XLAT_STYLE_VERBOSE
-                ? tprints_comment : tprints)(str);
+                ? tprints_comment : tprints_string)(str);
 }
 
 static void
@@ -78,7 +78,7 @@ print_rlim32_t(uint32_t lim) {
 		return;
 
 	(xlat_verbose(xlat_verbosity) == XLAT_STYLE_VERBOSE
-                ? tprints_comment : tprints)(str);
+                ? tprints_comment : tprints_string)(str);
 }
 
 static void
