@@ -38,7 +38,7 @@ DECL_NETLINK_ROUTE_DECODER(decode_netconfmsg)
 
 	const size_t offset = NLMSG_ALIGN(sizeof(ncm));
 	if (len > offset) {
-		tprints(", ");
+		tprint_array_next();
 		decode_nlattr(tcp, addr + offset, len - offset,
 			      rtnl_netconf_attrs, "NETCONFA_???",
 			      netconfmsg_nla_decoders,
