@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2016 Fabien Siron <fabien.siron@epita.fr>
  * Copyright (c) 2017 JingPiao Chen <chenjingpiao@gmail.com>
- * Copyright (c) 2016-2021 The strace developers.
+ * Copyright (c) 2016-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -94,6 +94,9 @@ static const nla_decoder_t fib_rule_hdr_nla_decoders[] = {
 	[FRA_IP_PROTO]			= decode_nla_ip_proto,
 	[FRA_SPORT_RANGE]		= decode_rule_port_range,
 	[FRA_DPORT_RANGE]		= decode_rule_port_range,
+	[FRA_DSCP]			= decode_nla_u8,
+	[FRA_FLOWLABEL]			= decode_nla_be32,
+	[FRA_FLOWLABEL_MASK]		= decode_nla_be32,
 };
 
 DECL_NETLINK_ROUTE_DECODER(decode_fib_rule_hdr)
