@@ -13,13 +13,21 @@
 static int
 do_signalfd(struct tcb *tcp, int flags_arg)
 {
-	/* NB: kernel requires arg[2] == NSIG_BYTES */
+	/* fd */
 	printfd(tcp, tcp->u_arg[0]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* NB: kernel requires arg[2] == NSIG_BYTES */
+	/* mask */
 	print_sigset_addr_len(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-	tprintf(", %" PRI_klu, tcp->u_arg[2]);
+	tprint_arg_next();
+
+	/* sizemask */
+	PRINT_VAL_U(tcp->u_arg[2]);
 	if (flags_arg >= 0) {
-		tprints(", ");
+		tprint_arg_next();
+
+		/* flags */
 		printflags(sfd_flags, tcp->u_arg[flags_arg], "SFD_???");
 	}
 
