@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022 The strace developers.
+ * Copyright (c) 2017-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -152,6 +152,22 @@ SYS_FUNC(statx)
 				tprint_struct_next();
 				PRINT_FIELD_U(stx, stx_dio_offset_align);
 			}
+			if (stx.stx_mask & STATX_SUBVOL) {
+				tprint_struct_next();
+				PRINT_FIELD_X(stx, stx_subvol);
+			}
+			if (stx.stx_attributes & STATX_ATTR_WRITE_ATOMIC) {
+				tprint_struct_next();
+				PRINT_FIELD_U(stx, stx_atomic_write_unit_min);
+				tprint_struct_next();
+				PRINT_FIELD_U(stx, stx_atomic_write_unit_max);
+				tprint_struct_next();
+				PRINT_FIELD_U(stx, stx_atomic_write_segments_max);
+			}
+			if (stx.stx_mask & STATX_DIO_READ_ALIGN) {
+				tprint_struct_next();
+				PRINT_FIELD_U(stx, stx_dio_read_offset_align);
+			}
 		} else {
 			tprint_struct_next();
 			tprint_more_data_follows();
