@@ -647,9 +647,17 @@ printsocket(struct tcb *tcp, int fd, const char *path)
 
 typedef bool (*scan_fdinfo_fn)(const char *value, void *data);
 
-static bool
-scan_fdinfo(pid_t pid_of_fd, int fd, const char *search_pfx,
-	    size_t search_pfx_len, scan_fdinfo_fn fn, void *data)
+struct scan_fdinfo {
+	const char *search_pfx;
+	size_t search_pfx_len;
+	scan_fdinfo_fn fn;
+	void *data;
+	size_t matches;
+};
+
+static size_t
+scan_fdinfo_lines(pid_t pid_of_fd, int fd, struct scan_fdinfo *fdinfo_search_array,
+		  size_t fdinfo_array_size)
 {
 	int proc_pid = 0;
 	translate_pid(NULL, pid_of_fd, PT_TID, &proc_pid);
@@ -665,21 +673,47 @@ scan_fdinfo(pid_t pid_of_fd, int fd, const char *search_pfx,
 
 	char *line = NULL;
 	size_t sz = 0;
-	bool result = false;
+	size_t matches = 0;
 
-	while (getline(&line, &sz, f) > 0) {
-		const char *value =
-			str_strip_prefix_len(line, search_pfx, search_pfx_len);
-		if (value != line && fn(value, data)) {
-			result = true;
-			break;
+	while (matches < fdinfo_array_size && getline(&line, &sz, f) > 0) {
+		for (size_t i = 0; i < fdinfo_array_size ; ++i) {
+			if (fdinfo_search_array[i].matches)
+				continue;
+
+			scan_fdinfo_fn fn = fdinfo_search_array[i].fn;
+			void *data = fdinfo_search_array[i].data;
+			const char *value =
+				str_strip_prefix_len(line, fdinfo_search_array[i].search_pfx,
+				                     fdinfo_search_array[i].search_pfx_len);
+			if (value != line && fn(value, data)) {
+				++fdinfo_search_array[i].matches;
+				++matches;
+				break;
+			}
 		}
 	}
 
 	free(line);
 	fclose(f);
 
-	return result;
+	return matches;
+}
+
+static inline bool
+scan_fdinfo(pid_t pid_of_fd, int fd, const char *search_pfx,
+	    size_t search_pfx_len, scan_fdinfo_fn fn, void *data)
+{
+	struct scan_fdinfo fdinfo_lines[] = {
+		{
+			.search_pfx = search_pfx,
+			.search_pfx_len = search_pfx_len,
+			.fn = fn,
+			.data = data
+		}
+	};
+
+	return scan_fdinfo_lines(pid_of_fd, fd, fdinfo_lines,
+				 ARRAY_SIZE(fdinfo_lines)) > 0;
 }
 
 static bool
@@ -858,6 +892,107 @@ printsignalfd(pid_t pid_of_fd, int fd, const char *path)
 			   print_fdinfo_sigmask, NULL);
 }
 
+static bool
+parse_fdinfo_efd_semaphore(const char *value, void *data)
+{
+	int *efd_semaphore = data;
+	*efd_semaphore = string_to_uint_ex(value, NULL, INT_MAX, "\n");
+	return true;
+}
+
+static bool
+parse_fdinfo_efd_id(const char *value, void *data)
+{
+	int *efd_id = data;
+	*efd_id = string_to_uint_ex(value, NULL, INT_MAX, "\n");
+	return true;
+}
+
+static bool
+parse_fdinfo_efd_counter(const char *value, void *data)
+{
+	char *ptr = (char *) value;
+
+	ptr += strspn(ptr, " \t");
+	ptr[strcspn(ptr, "\n")] = '\0';
+
+	if (*ptr == '\0')
+		ptr = NULL;
+
+	*(char **) data = xstrdup(ptr);
+	return true;
+}
+
+static bool
+printeventfd(pid_t pid_of_fd, int fd, const char *path)
+{
+	static const char eventfd_path[] = "anon_inode:[eventfd]";
+	/* Linux kernel commit v3.8-rc1~74^2~8 */
+	static const char efd_counter_pfx[] = "eventfd-count:";
+	/* Linux kernel commit v5.2-rc1~62^2~38 */
+	static const char efd_id_pfx[] = "eventfd-id:";
+	/* Linux kernel commit v6.5-rc1~246^2~5 */
+	static const char efd_semaphore_pfx[] = "eventfd-semaphore:";
+
+	if (strcmp(path, eventfd_path))
+		return false;
+
+	char *efd_counter = NULL;
+	int efd_id = -1;
+	int efd_semaphore = -1;
+
+	struct scan_fdinfo fdinfo_lines[] = {
+		{
+			.search_pfx = efd_counter_pfx,
+			.search_pfx_len = sizeof(efd_counter_pfx) - 1,
+			.fn = parse_fdinfo_efd_counter,
+			.data = &efd_counter
+		},
+		{
+			.search_pfx = efd_id_pfx,
+			.search_pfx_len = sizeof(efd_id_pfx) - 1,
+			.fn = parse_fdinfo_efd_id,
+			.data = &efd_id
+		},
+		{
+			.search_pfx = efd_semaphore_pfx,
+			.search_pfx_len = sizeof(efd_semaphore_pfx) - 1,
+			.fn = parse_fdinfo_efd_semaphore,
+			.data = &efd_semaphore
+		}
+	};
+
+	scan_fdinfo_lines(pid_of_fd, fd, fdinfo_lines, ARRAY_SIZE(fdinfo_lines));
+
+	if (efd_counter) {
+		tprint_associated_info_begin();
+		tprint_struct_begin();
+		tprints_field_name("eventfd-count");
+		if (efd_counter[0] != '0')
+			tprints_string("0x");
+		tprints_string(efd_counter);
+		free(efd_counter);
+
+		if (efd_id != -1) {
+			tprint_struct_next();
+			tprints_field_name("eventfd-id");
+			PRINT_VAL_U(efd_id);
+
+			if (efd_semaphore != -1) {
+				tprint_struct_next();
+				tprints_field_name("eventfd-semaphore");
+				PRINT_VAL_U(efd_semaphore);
+			}
+		}
+
+		tprint_struct_end();
+		tprint_associated_info_end();
+	} else
+		print_string_in_angle_brackets(path);
+
+	return true;
+}
+
 static void
 print_quoted_string_in_angle_brackets(const char *str, const bool deleted)
 {
@@ -886,6 +1021,9 @@ printfd_pid_with_finfo(struct tcb *tcp, pid_t pid, int fd, const struct finfo *f
 		if (is_number_in_set(DECODE_FD_DEV, decode_fd_set) &&
 		    printdev(tcp, fd, path, finfo))
 			goto printed;
+		if (is_number_in_set(DECODE_FD_EVENTFD, decode_fd_set) &&
+		    printeventfd(pid, fd, path))
+			goto printed;
 		if (is_number_in_set(DECODE_FD_PIDFD, decode_fd_set) &&
 		    printpidfd(pid, fd, path))
 			goto printed;
@@ -1227,8 +1365,9 @@ print_quoted_cstring(const char *str, unsigned int size)
 }
 
 /*
- * Print path string specified by address `addr' and length `n'.
- * If path length exceeds `n', append `...' to the output.
+ * Print path string specified by address `addr' and length `n',
+ * including the terminating NUL.
+ * If path length exceeds `n - 1', append `...' to the output.
  *
  * Returns the result of umovestr.
  */
@@ -1243,17 +1382,19 @@ printpathn(struct tcb *const tcp, const kernel_ulong_t addr, unsigned int n)
 		return -1;
 	}
 
-	/* Cap path length to the path buffer size */
-	if (n > sizeof(path) - 1)
-		n = sizeof(path) - 1;
+	/* Cap path size to the path buffer size */
+	if (n > sizeof(path))
+		n = sizeof(path);
 
-	/* Fetch one byte more to find out whether path length > n. */
-	nul_seen = umovestr(tcp, addr, n + 1, path);
+	/*
+	 * Fetch including the terminating NUL to find out
+	 * whether path length >= n.
+	 */
+	nul_seen = umovestr(tcp, addr, n, path);
 	if (nul_seen < 0)
 		printaddr(addr);
 	else {
-		path[n++] = !nul_seen;
-		print_quoted_cstring(path, n);
+		print_quoted_cstring(path, (unsigned int) nul_seen ?: n);
 
 		if (nul_seen)
 			selinux_printfilecon(tcp, path);
@@ -1266,7 +1407,7 @@ int
 printpath(struct tcb *const tcp, const kernel_ulong_t addr)
 {
 	/* Size must correspond to char path[] size in printpathn */
-	return printpathn(tcp, addr, PATH_MAX - 1);
+	return printpathn(tcp, addr, PATH_MAX);
 }
 
 /*
@@ -1774,7 +1915,7 @@ print_array_ex(struct tcb *const tcp,
 	}
 
 	const kernel_ulong_t abbrev_end =
-		(abbrev(tcp) && max_strlen < nmemb) ?
+		sequence_truncation_needed(tcp, nmemb) ?
 			start_addr + elem_size * max_strlen : end_addr;
 	kernel_ulong_t cur;
 	kernel_ulong_t idx = 0;
@@ -1937,3 +2078,9 @@ read_int_from_file(const char *const fname, int *const pvalue)
 	*pvalue = (int) lval;
 	return 0;
 }
+
+bool
+sequence_truncation_needed(const struct tcb *tcp, unsigned int len)
+{
+	return abbrev(tcp) && len > max_strlen;
+}
