@@ -1,4 +1,4 @@
-#define PIDFD_PATH "<anon_inode:[pidfd]>"
+#define PRINT_PIDFD_PATH 1
 #define FD0_PATH "</dev/full>"
 #define SKIP_IF_PROC_IS_UNAVAILABLE skip_if_unavailable("/proc/self/fd/")
 
