@@ -12,7 +12,7 @@
 #include <pthread.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <asm/unistd.h>
+#include "scno.h"
 
 static pid_t leader;
 
