@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2015-2018 Dmitry V. Levin <ldv@altlinux.org>
- * Copyright (c) 2015-2019 The strace developers.
+ * Copyright (c) 2015-2018 Dmitry V. Levin <ldv@strace.io>
+ * Copyright (c) 2015-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -22,8 +22,8 @@ main(void)
 	const off_t len = 0xdefaceddeadbeefULL;
 
 	int rc = truncate(fname, len);
-	printf("truncate64(\"%s\", %llu) = %d %s (%m)\n",
-	       qname, (unsigned long long) len, rc, errno2name());
+	printf("truncate64(\"%s\", %llu) = %s\n",
+	       qname, (unsigned long long) len, sprintrc(rc));
 
 	puts("+++ exited with 0 +++");
 	return 0;
