@@ -1,7 +1,7 @@
 /*
  * Check decoding of clone3 syscall.
  *
- * Copyright (c) 2019-2020 The strace developers.
+ * Copyright (c) 2019-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -17,19 +17,12 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
-#ifdef HAVE_LINUX_SCHED_H
-# include <linux/sched.h>
-#endif
+#include <linux/sched.h>
 
 #ifdef HAVE_STRUCT_USER_DESC
 # include <asm/ldt.h>
 #endif
 
-#define XLAT_MACROS_ONLY
-# include "xlat/clone_flags.h"
-# include "xlat/clone3_flags.h"
-#undef XLAT_MACROS_ONLY
-
 #include "scno.h"
 
 #ifndef VERBOSE
@@ -39,26 +32,6 @@
 # define RETVAL_INJECTED 0
 #endif
 
-struct test_clone_args {
-	uint64_t flags;
-	uint64_t pidfd;
-	uint64_t child_tid;
-	uint64_t parent_tid;
-	uint64_t exit_signal;
-	uint64_t stack;
-	uint64_t stack_size;
-	uint64_t tls;
-	uint64_t set_tid;
-	uint64_t set_tid_size;
-	uint64_t cgroup;
-};
-
-#ifdef HAVE_STRUCT_CLONE_ARGS_CGROUP
-typedef struct clone_args struct_clone_args;
-#else
-typedef struct test_clone_args struct_clone_args;
-#endif
-
 enum validity_flag_bits {
 	STRUCT_VALID_BIT,
 	PIDFD_VALID_BIT,
@@ -143,7 +116,7 @@ do_clone3_(void *args, kernel_ulong_t size, uint64_t possible_errors, int line)
 	if (!rc)
 		_exit(child_exit_status);
 
-	if (rc > 0 && ((struct_clone_args *) args)->exit_signal)
+	if (rc > 0 && ((struct clone_args *) args)->exit_signal)
 		wait_cloned(rc);
 #endif
 
@@ -153,7 +126,7 @@ do_clone3_(void *args, kernel_ulong_t size, uint64_t possible_errors, int line)
 #define do_clone3(args_, size_, errors_) \
 	do_clone3_((args_), (size_), (errors_), __LINE__)
 
-static inline void
+static void
 print_addr64(const char *pfx, uint64_t addr)
 {
 	if (addr)
@@ -214,8 +187,8 @@ print_set_tid(uint64_t set_tid, uint64_t set_tid_size)
 	printf(", set_tid_size=%" PRIu64, set_tid_size);
 }
 
-static inline void
-print_clone3(struct_clone_args *const arg, long rc, kernel_ulong_t sz,
+static void
+print_clone3(struct clone_args *const arg, long rc, kernel_ulong_t sz,
 	     enum validity_flags valid,
 	     const char *flags_str, const char *es_str)
 {
@@ -260,11 +233,11 @@ print_clone3(struct_clone_args *const arg, long rc, kernel_ulong_t sz,
 	if (arg->flags & CLONE_SETTLS)
 		print_tls("tls=", arg->tls, valid);
 
-	if (sz >= offsetofend(struct_clone_args, set_tid_size) &&
+	if (sz >= offsetofend(struct clone_args, set_tid_size) &&
 	    (arg->set_tid || arg->set_tid_size))
 		print_set_tid(arg->set_tid, arg->set_tid_size);
 
-	if (sz > offsetof(struct_clone_args, cgroup) &&
+	if (sz > offsetof(struct clone_args, cgroup) &&
 	    (arg->cgroup || arg->flags & CLONE_INTO_CGROUP))
 		printf(", cgroup=%" PRIu64, (uint64_t) arg->cgroup);
 
@@ -308,7 +281,7 @@ int
 main(int argc, char *argv[])
 {
 	static const struct {
-		struct_clone_args args;
+		struct clone_args args;
 		uint64_t possible_errors;
 		enum validity_flags vf;
 		const char *flags_str;
@@ -334,15 +307,15 @@ main(int argc, char *argv[])
 		    .set_tid_size = MAX_SET_TID_SIZE + 1 },
 			ERR(E2BIG) | ERR(EINVAL), 0, "0", "0" },
 	};
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_clone_args, arg);
-	const size_t arg1_size = offsetofend(struct_clone_args, tls);
-	struct_clone_args *const arg1 = tail_alloc(arg1_size);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct clone_args, arg);
+	const size_t arg1_size = offsetofend(struct clone_args, tls);
+	struct clone_args *const arg1 = tail_alloc(arg1_size);
 	const size_t arg2_size = sizeof(*arg) + 8;
-	struct_clone_args *const arg2 = tail_alloc(arg2_size);
+	struct clone_args *const arg2 = tail_alloc(arg2_size);
 	TAIL_ALLOC_OBJECT_CONST_PTR(int, pidfd);
 	TAIL_ALLOC_OBJECT_CONST_PTR(int, child_tid);
 	TAIL_ALLOC_OBJECT_CONST_PTR(int, parent_tid);
-	int *const tids = tail_alloc(sizeof(*tids) * MAX_SET_TID_SIZE);
+	TAIL_ALLOC_OBJECT_CONST_ARR(int, tids, MAX_SET_TID_SIZE);
 	long rc;
 
 #if defined HAVE_STRUCT_USER_DESC
@@ -429,18 +402,12 @@ main(int argc, char *argv[])
 	rc = do_clone3(arg2, sizeof(*arg2) + 8, ERR(E2BIG));
 	printf("clone3({flags=0, exit_signal=%llu, stack=NULL, stack_size=0"
 	       ", /* bytes %zu..%zu */ "
-#if WORDS_BIGENDIAN
-	       "\"\\xfa\\xce\\xfe\\xed\\xde\\xad\\xc0\\xde\""
-#else
-	       "\"\\xde\\xc0\\xad\\xde\\xed\\xfe\\xce\\xfa\""
-#endif
+	       BE_LE("\"\\xfa\\xce\\xfe\\xed\\xde\\xad\\xc0\\xde\"",
+		     "\"\\xde\\xc0\\xad\\xde\\xed\\xfe\\xce\\xfa\"")
 #if RETVAL_INJECTED
 	       "} => {/* bytes %zu..%zu */ "
-# if WORDS_BIGENDIAN
-	       "\"\\xfa\\xce\\xfe\\xed\\xde\\xad\\xc0\\xde\""
-# else
-	       "\"\\xde\\xc0\\xad\\xde\\xed\\xfe\\xce\\xfa\""
-# endif
+	       BE_LE("\"\\xfa\\xce\\xfe\\xed\\xde\\xad\\xc0\\xde\"",
+		     "\"\\xde\\xc0\\xad\\xde\\xed\\xfe\\xce\\xfa\"")
 #endif /* RETVAL_INJECTED */
 	       "}, %zu) = %s" INJ_STR,
 	       0xdeadface00000000ULL | SIGCHLD,
@@ -533,10 +500,11 @@ main(int argc, char *argv[])
 		rc = do_clone3(arg, 64, ERR(EINVAL));
 		rc_str = sprintrc(rc);
 		printf("clone3({flags=%s, %s=NULL"
-		       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+		       ", exit_signal=" XLAT_FMT_U
 		       ", stack=0xface1e55beeff00d"
 		       ", stack_size=0xcaffeedefacedca7}",
-		       flag_str, pid_fields[i].field_name);
+		       flag_str, pid_fields[i].field_name,
+		       XLAT_ARGS(SIGCHLD));
 #if RETVAL_INJECTED
 		if (pid_fields[i].deref_exiting)
 			printf(" => {%s=NULL}", pid_fields[i].field_name);
@@ -547,11 +515,12 @@ main(int argc, char *argv[])
 		rc = do_clone3(arg, 64, ERR(EINVAL));
 		rc_str = sprintrc(rc);
 		printf("clone3({flags=%s, %s=%p"
-		       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+		       ", exit_signal=" XLAT_FMT_U
 		       ", stack=0xface1e55beeff00d"
 		       ", stack_size=0xcaffeedefacedca7}",
 		       flag_str, pid_fields[i].field_name,
-		       pid_fields[i].ptr + 1);
+		       pid_fields[i].ptr + 1,
+		       XLAT_ARGS(SIGCHLD));
 #if RETVAL_INJECTED
 		if (pid_fields[i].deref_exiting)
 			printf(" => {%s=%p}",
@@ -563,11 +532,12 @@ main(int argc, char *argv[])
 		rc = do_clone3(arg, 64, ERR(EINVAL));
 		rc_str = sprintrc(rc);
 		printf("clone3({flags=%s, %s=%p"
-		       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+		       ", exit_signal=" XLAT_FMT_U
 		       ", stack=0xface1e55beeff00d"
 		       ", stack_size=0xcaffeedefacedca7}",
 		       flag_str, pid_fields[i].field_name,
-		       pid_fields[i].ptr);
+		       pid_fields[i].ptr,
+		       XLAT_ARGS(SIGCHLD));
 #if RETVAL_INJECTED
 		if (pid_fields[i].deref_exiting)
 			printf(" => {%s=[%d]}",
@@ -580,25 +550,25 @@ main(int argc, char *argv[])
 	rc = do_clone3(arg, 64, ERR(EINVAL));
 	printf("clone3({flags="
 	       XLAT_KNOWN(0xbad0000000080001, "CLONE_SETTLS|0xbad0000000000001")
-	       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+	       ", exit_signal=" XLAT_FMT_U
 	       ", stack=0xface1e55beeff00d"
 	       ", stack_size=0xcaffeedefacedca7, tls=NULL}, 64) = %s" INJ_STR,
-	       sprintrc(rc));
+	       XLAT_ARGS(SIGCHLD), sprintrc(rc));
 
 	arg->tls = (uintptr_t) (tls + 1);
 	rc = do_clone3(arg, 64, ERR(EINVAL));
 	printf("clone3({flags="
 	       XLAT_KNOWN(0xbad0000000080001, "CLONE_SETTLS|0xbad0000000000001")
-	       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+	       ", exit_signal=" XLAT_FMT_U
 	       ", stack=0xface1e55beeff00d"
 	       ", stack_size=0xcaffeedefacedca7, tls=%p}, 64) = %s" INJ_STR,
-	       tls + 1, sprintrc(rc));
+	       XLAT_ARGS(SIGCHLD), tls + 1, sprintrc(rc));
 
 	arg->tls = (uintptr_t) tls;
 	rc = do_clone3(arg, 64, ERR(EINVAL));
 	printf("clone3({flags="
 	       XLAT_KNOWN(0xbad0000000080001, "CLONE_SETTLS|0xbad0000000000001")
-	       ", exit_signal=" XLAT_KNOWN(SIGCHLD, "SIGCHLD")
+	       ", exit_signal=" XLAT_FMT_U
 	       ", stack=0xface1e55beeff00d, stack_size=0xcaffeedefacedca7, tls="
 #if defined HAVE_STRUCT_USER_DESC && defined __i386__
 	       "{entry_number=2206368128, base_addr=0x87868584"
@@ -608,6 +578,7 @@ main(int argc, char *argv[])
 	       "%p"
 #endif
 	       "}, 64) = %s" INJ_STR,
+	       XLAT_ARGS(SIGCHLD),
 #if !defined HAVE_STRUCT_USER_DESC || !defined __i386__
 	       tls,
 #endif
