@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2015-2021 The strace developers.
+ * Copyright (c) 2015-2022 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -15,6 +15,10 @@
 # define TEST_SYSCALL_STR "fcntl"
 # include "fcntl-common.c"
 
+/* We can't have nice things because clang */
+# if (defined(F_SETLK64) && (!defined(F_GETOWN_EX) || F_GETOWN_EX != F_SETLK64)) || \
+     (defined(F_SETLKW64) && (!defined(__mips64) || F_GETLK != F_SETLKW64)) || \
+     (defined(F_GETLK64) && (!defined(F_SETOWN_EX) || F_SETOWN_EX != F_GETLK64))
 static void
 test_flock64_undecoded(const int cmd, const char *name)
 {
@@ -28,6 +32,7 @@ test_flock64_undecoded(const int cmd, const char *name)
 	printf("%s(0, %s, %p) = %s\n",
 	       TEST_SYSCALL_STR, name, &fl, errstr);
 }
+# endif
 
 # define TEST_FLOCK64_UNDECODED(cmd) test_flock64_undecoded(cmd, #cmd)
 
@@ -38,14 +43,14 @@ test_flock64_lk64(void)
  * F_[GS]ETOWN_EX had conflicting values with F_[GS]ETLK64
  * in kernel revisions v2.6.32-rc1~96..v2.6.32-rc7~23.
  */
-# if !defined(F_GETOWN_EX) || F_GETOWN_EX != F_SETLK64
+# if defined(F_SETLK64) && (!defined(F_GETOWN_EX) || F_GETOWN_EX != F_SETLK64)
 	TEST_FLOCK64_UNDECODED(F_SETLK64);
 # endif
 /* F_GETLK and F_SETLKW64 have conflicting values on mips64 */
-# if !defined(__mips64) || F_GETLK != F_SETLKW64
+# if defined(F_SETLKW64) && (!defined(__mips64) || F_GETLK != F_SETLKW64)
 	TEST_FLOCK64_UNDECODED(F_SETLKW64);
 # endif
-# if !defined(F_SETOWN_EX) || F_SETOWN_EX != F_GETLK64
+# if defined(F_GETLK64) && (!defined(F_SETOWN_EX) || F_SETOWN_EX != F_GETLK64)
 	TEST_FLOCK64_UNDECODED(F_GETLK64);
 # endif
 }
