@@ -2,7 +2,7 @@
  * Check decoding of ptrace syscall.
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2016-2022 The strace developers.
+ * Copyright (c) 2016-2025 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -31,8 +31,8 @@
 #include "xlat.h"
 #define XLAT_MACROS_ONLY
 # include "xlat/elf_em.h"
+# include "xlat/audit_arch.h"
 #undef XLAT_MACROS_ONLY
-#include "xlat/audit_arch.h"
 
 #define NULL_FD 23
 #define NULL_STR "/dev/null"
@@ -41,6 +41,10 @@
 # define NULL_FD_STR ""
 #endif
 
+#ifndef SKIP_IF_PROC_IS_UNAVAILABLE
+# define SKIP_IF_PROC_IS_UNAVAILABLE
+#endif
+
 static const char null_path[] = "/dev/null";
 
 #if SIZEOF_LONG > 4
@@ -147,7 +151,7 @@ test_peeksiginfo(int pid, const unsigned long bad_request)
 
 	const unsigned int nsigs = 4;
 	const uid_t uid = geteuid();
-	siginfo_t *sigs = tail_alloc(sizeof(*sigs) * nsigs);
+	TAIL_ALLOC_OBJECT_CONST_ARR(siginfo_t, sigs, nsigs);
 
 	psi->off = 0;
 	psi->flags = 0;
@@ -1694,6 +1698,8 @@ main(void)
 		(unsigned long) 0xdeadcafefffff00dULL;
 	const int pid = getpid();
 
+	SKIP_IF_PROC_IS_UNAVAILABLE;
+
 	int null_fd = open(null_path, O_RDONLY);
 	if (null_fd < 0)
 		perror_msg_and_fail("open(\"%s\")", null_path);
@@ -1836,7 +1842,7 @@ main(void)
 		{ ARG_XLAT_KNOWN(0x4, "TRAP_HWBKPT") },
 #endif
 		{ ARG_XLAT_KNOWN(0x5, "TRAP_UNK"), ""
-#ifdef __alpha__
+#if defined __alpha__ && defined HAVE_SIGINFO_T_SI_TRAPNO
 		  ", si_trapno=0 /* GEN_??? */"
 		},
 		{ ARG_XLAT_KNOWN(0x5, "TRAP_UNK"),
@@ -1847,7 +1853,7 @@ main(void)
 		  ", si_trapno=-26 /* GEN_??? */" },
 		{ ARG_XLAT_KNOWN(0x5, "TRAP_UNK"), ""
 		  ", si_trapno=-1234567890 /* GEN_??? */"
-#endif /* __alpha__ */
+#endif /* __alpha__ && HAVE_SIGINFO_T_SI_TRAPNO */
 		},
 		{ ARG_XLAT_KNOWN(0x6, "TRAP_PERF"), ""
 #ifdef HAVE_SIGINFO_T_SI_PERF_DATA
@@ -1917,7 +1923,7 @@ main(void)
 
 		switch (sip->si_code) {
 		case 5: /* TRAP_UNK */
-#ifdef __alpha__
+#if defined __alpha__ && defined HAVE_SIGINFO_T_SI_TRAPNO
 			sip->si_trapno = trap_unk_vecs[trap_unk_pos];
 #endif
 			trap_unk_pos = (trap_unk_pos + 1)
@@ -1981,7 +1987,7 @@ main(void)
 	do_ptrace(PTRACE_SETSIGINFO, pid, bad_request, (uintptr_t) sip);
 	printf("ptrace(" XLAT_FMT ", %d, %#lx, {si_signo=" XLAT_FMT_U
 	       ", si_code=" XLAT_FMT ", si_errno=" XLAT_FMT_U ", si_addr=%p"
-#ifdef __alpha__
+#if defined __alpha__ && defined HAVE_SIGINFO_T_SI_TRAPNO
 	       ", si_trapno=" XLAT_KNOWN(-7, "GEN_FLTINE")
 #endif
 	       "}) = %s\n",
@@ -2004,9 +2010,9 @@ main(void)
 		},
 #endif
 		{ ARG_XLAT_KNOWN(0x5, "BUS_MCEERR_AO"),
-# if !defined(BUS_OPFETCH) && defined(HAVE_SIGINFO_T_SI_ADDR_LSB)
+#if !defined(BUS_OPFETCH) && defined(HAVE_SIGINFO_T_SI_ADDR_LSB)
 		  ", si_addr_lsb=0xdead"
-# endif
+#endif
 		},
 		{ ARG_STR(0x6) },
 		{ ARG_STR(0x499602d2) },
@@ -2067,7 +2073,7 @@ main(void)
 		{ ARG_XLAT_KNOWN(0x3, "SEGV_BNDERR"),
 		  ", si_lower=0x" UP64BIT("beefface") "cafedead"
 		  ", si_upper=0x" UP64BIT("badc0ded") "dadfaced",
-#endif /* HAVE_SIGINFO_T_SI_LOWER */
+# endif /* HAVE_SIGINFO_T_SI_LOWER */
 		},
 #endif /* SEGV_STACKFLOW */
 #ifdef __SEGV_PSTKOVF
@@ -2087,7 +2093,8 @@ main(void)
 		{ ARG_XLAT_KNOWN(0x7, "SEGV_ADIPERR") },
 		{ ARG_XLAT_KNOWN(0x8, "SEGV_MTEAERR") },
 		{ ARG_XLAT_KNOWN(0x9, "SEGV_MTESERR") },
-		{ ARG_STR(0xa) },
+		{ ARG_XLAT_KNOWN(0xa, "SEGV_CPERR") },
+		{ ARG_STR(0xb) },
 		{ ARG_STR(0x499602d2) },
 	};
 	uint32_t segv_pkey_vecs[] = { 0, 1234567890, 3141592653U };
@@ -2177,11 +2184,11 @@ main(void)
 	do_ptrace(PTRACE_SETSIGINFO, pid, bad_request, (uintptr_t) sip);
 	printf("ptrace(" XLAT_FMT ", %d, %#lx, {si_signo=" XLAT_FMT_U
 	       ", si_code=" XLAT_KNOWN(0x1, "SYS_SECCOMP") ", si_errno=%u"
-	       ", si_call_addr=NULL, si_syscall=" XLAT_FMT_U ", si_arch=%s}"
-	       ") = %s\n",
+	       ", si_call_addr=NULL, si_syscall=" XLAT_FMT_U ", si_arch="
+	       XLAT_FMT "}) = %s\n",
 	       XLAT_ARGS(PTRACE_SETSIGINFO), pid, bad_request,
 	       XLAT_ARGS(SIGSYS), sip->si_errno, XLAT_ARGS(__NR_read),
-	       sprintxval(audit_arch, CUR_AUDIT_ARCH, "AUDIT_ARCH_???"),
+	       XLAT_SEL(CUR_AUDIT_ARCH, CUR_AUDIT_ARCH_STR),
 	       errstr);
 # endif
 # if defined(PERS0_AUDIT_ARCH)
@@ -2192,10 +2199,10 @@ main(void)
 	printf("ptrace(" XLAT_FMT ", %d, %#lx, {si_signo=" XLAT_FMT_U
 	       ", si_code=" XLAT_KNOWN(0x1, "SYS_SECCOMP") ", si_errno=%u"
 	       ", si_call_addr=NULL, si_syscall=%u" NRAW(" /* gettid */")
-	       ", si_arch=%s}) = %s\n",
+	       ", si_arch=" XLAT_FMT "}) = %s\n",
 	       XLAT_ARGS(PTRACE_SETSIGINFO), pid, bad_request,
 	       XLAT_ARGS(SIGSYS), sip->si_errno, PERS0__NR_gettid,
-	       sprintxval(audit_arch, PERS0_AUDIT_ARCH, "AUDIT_ARCH_???"),
+	       XLAT_SEL(PERS0_AUDIT_ARCH, PERS0_AUDIT_ARCH_STR),
 	       errstr);
 # endif
 # if defined(M32_AUDIT_ARCH)
@@ -2206,10 +2213,10 @@ main(void)
 	printf("ptrace(" XLAT_FMT ", %d, %#lx, {si_signo=" XLAT_FMT_U
 	       ", si_code=" XLAT_KNOWN(0x1, "SYS_SECCOMP") ", si_errno=%u"
 	       ", si_call_addr=NULL, si_syscall=%u" NRAW(" /* gettid */")
-	       ", si_arch=%s}) = %s\n",
+	       ", si_arch=" XLAT_FMT "}) = %s\n",
 	       XLAT_ARGS(PTRACE_SETSIGINFO), pid, bad_request,
 	       XLAT_ARGS(SIGSYS), sip->si_errno, M32__NR_gettid,
-	       sprintxval(audit_arch, M32_AUDIT_ARCH, "AUDIT_ARCH_???"),
+	       XLAT_SEL(M32_AUDIT_ARCH, M32_AUDIT_ARCH_STR),
 	       errstr);
 # endif
 # if defined(MX32_AUDIT_ARCH)
@@ -2220,10 +2227,10 @@ main(void)
 	printf("ptrace(" XLAT_FMT ", %d, %#lx, {si_signo=" XLAT_FMT_U
 	       ", si_code=" XLAT_KNOWN(0x1, "SYS_SECCOMP") ", si_errno=%u"
 	       ", si_call_addr=NULL, si_syscall=%u" NRAW(" /* gettid */")
-	       ", si_arch=%s}) = %s\n",
+	       ", si_arch=" XLAT_FMT "}) = %s\n",
 	       XLAT_ARGS(PTRACE_SETSIGINFO), pid, bad_request,
 	       XLAT_ARGS(SIGSYS), sip->si_errno, MX32__NR_gettid,
-	       sprintxval(audit_arch, MX32_AUDIT_ARCH, "AUDIT_ARCH_???"),
+	       XLAT_SEL(MX32_AUDIT_ARCH, MX32_AUDIT_ARCH_STR),
 	       errstr);
 # endif
 #endif
