@@ -2,7 +2,7 @@
  * Check decoding of set_mempolicy syscall.
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2016-2022 The strace developers.
+ * Copyright (c) 2016-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -69,6 +69,9 @@ static struct {
 	{ ARG_STR(0x5),
 	  "0x5 /* MPOL_PREFERRED_MANY */",
 	  "MPOL_PREFERRED_MANY" },
+	{ ARG_STR(0x6),
+	  "0x6 /* MPOL_WEIGHTED_INTERLEAVE */",
+	  "MPOL_WEIGHTED_INTERLEAVE" },
 	{ ARG_STR(0x8000),
 	  "0x8000 /* MPOL_DEFAULT|MPOL_F_STATIC_NODES */",
 	  "MPOL_DEFAULT|MPOL_F_STATIC_NODES" },
@@ -83,9 +86,9 @@ static struct {
 		"|MPOL_F_NUMA_BALANCING */",
 	  "MPOL_INTERLEAVE|MPOL_F_STATIC_NODES|MPOL_F_RELATIVE_NODES"
 		"|MPOL_F_NUMA_BALANCING" },
-	{ ARG_STR(0x6),
-	  "0x6 /* MPOL_??? */",
-	  "0x6 /* MPOL_??? */" },
+	{ ARG_STR(0x7),
+	  "0x7 /* MPOL_??? */",
+	  "0x7 /* MPOL_??? */" },
 	{ ARG_STR(0xffff1fff),
 	  "0xffff1fff /* MPOL_??? */",
 	  "0xffff1fff /* MPOL_??? */" },
