@@ -1,8 +1,8 @@
 /*
- * This file is part of splice strace test.
+ * Check decoding of splice syscall.
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2016-2021 The strace developers.
+ * Copyright (c) 2016-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -11,10 +11,8 @@
 #include "tests.h"
 #include "scno.h"
 
-#if defined __NR_splice
-
-# include <stdio.h>
-# include <unistd.h>
+#include <stdio.h>
+#include <unistd.h>
 
 int
 main(void)
@@ -30,17 +28,11 @@ main(void)
 
 	long rc = syscall(__NR_splice,
 			  fd_in, off_in, fd_out, off_out, len, flags);
-	printf("splice(%d, [%lld], %d, [%lld], %zu, %s) = %ld %s (%m)\n",
+	printf("splice(%d, [%lld], %d, [%lld], %zu, %s) = %s\n",
 	       (int) fd_in, *off_in, (int) fd_out, *off_out, len,
 	       "SPLICE_F_MOVE|SPLICE_F_NONBLOCK|SPLICE_F_MORE|SPLICE_F_GIFT",
-	       rc, errno2name());
+	       sprintrc(rc));
 
 	puts("+++ exited with 0 +++");
 	return 0;
 }
-
-#else
-
-SKIP_MAIN_UNDEFINED("__NR_splice")
-
-#endif
