@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 # Copyright (c) 2011-2016 Dmitry V. Levin <ldv@strace.io>
-# Copyright (c) 2011-2023 The strace developers.
+# Copyright (c) 2011-2024 The strace developers.
 # All rights reserved.
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
@@ -32,6 +32,30 @@ sed_slash_escape()
 	printf "%s" "$*" | sed 's/[/]/\\&/g'
 }
 
+# Calculate an integer approximation of the square root of $1:
+# returns 0 and prints the largest number the square of which is no greater
+# than $1 if $1 is non-negative, returns 1 and prints nothing otherwise.
+sq_root()
+{
+	# Handling the negative cases
+	[ 0 -le "$1" ] || return 1;
+
+	# Handling the 0 and 1 cases
+	[ 1 -lt "$1" ] || { echo "$1"; return 0; }
+
+	local upper="$1"
+	local lower=1
+	local cur
+
+	while :; do
+		cur="$(( (lower + upper) / 2 ))"
+		[ "$((cur * cur))" -ne "$1" ] || { echo "$cur"; return 0; }
+		[ "$((cur * cur))" -lt "$1" ] || { upper="$cur"; continue; }
+		[ "$cur" -ne "$lower" ] || { echo "$cur"; return 0; }
+		lower="$cur"
+	done
+}
+
 # get_config_str OPTION
 #
 # Returns the value of OPTION from config.h (path to which set
@@ -52,7 +76,7 @@ get_config_option()
 	local opt
 	opt=$(sed -E -n 's/#define[[:space:]]*'"$1"'[[:space:]]*([0-9]+)$/\1/p' \
 		"$CONFIG_H")
-	if [ -n "$opt" -a "$opt" -ne 0 ]; then
+	if [ -n "$opt" ] && [ "$opt" -ne 0 ]; then
 		printf "%s" "$2"
 	else
 		printf "%s" "${3-}"
@@ -280,9 +304,9 @@ run_strace_match_diff()
 	sed_cmd='p'
 
 	args="$*"
-	[ -n "$args" -a \( -z "${args##*-e trace=*}" -o \
-			   -z "${args##*-etrace=*}" -o \
-			   -z "${args##*--trace=*}" \) ] ||
+	[ -n "$args" ] && [ -z "${args##*-e trace=*}" -o \
+			    -z "${args##*-etrace=*}" -o \
+			    -z "${args##*--trace=*}" ] ||
 		set -- -e trace="$NAME" "$@"
 
 	set -- "$@" END_OF_ARGUMENTS
@@ -323,9 +347,9 @@ run_strace_match_diff()
 run_strace_match_grep()
 {
 	args="$*"
-	[ -n "$args" -a \( -z "${args##*-e trace=*}" -o \
-			   -z "${args##*-etrace=*}" -o \
-			   -z "${args##*--trace=*}" \) ] ||
+	[ -n "$args" ] && [ -z "${args##*-e trace=*}" -o \
+			    -z "${args##*-etrace=*}" -o \
+			    -z "${args##*--trace=*}" ] ||
 		set -- -e trace="$NAME" "$@"
 	run_prog > /dev/null
 	run_strace "$@" $args > "$EXP"
@@ -358,6 +382,17 @@ require_min_kernel_version_or_skip()
 		skip_ "the kernel release $uname_r is not $1 or newer"
 }
 
+# Usage: require_max_kernel_version_or_skip 6.11
+require_max_kernel_version_or_skip()
+{
+	local uname_r
+	uname_r="$(uname -r)"
+
+	[ "$(kernel_version_code "$uname_r")" -lt \
+	  "$(kernel_version_code "$1")" ] ||
+		skip_ "the kernel release $uname_r is $1 or newer"
+}
+
 # Usage: require_min_nproc 2
 require_min_nproc()
 {
@@ -433,9 +468,9 @@ test_pure_prog_set()
 	} < /dev/null; done
 }
 
-# Run strace against list of programs put in "$NAME.in" and then against the
-# rest of pure_executables.list with the expectation of empty output in the
-# latter case.
+# Run strace against list of programs put in "$NAME.in" and then against
+# a random selection of the binaries from the rest of pure_executables.list
+# with the expectation of empty output in the latter case.
 #
 # Usage: source this file after init.sh and call:
 #   test_trace_expr subtrahend_regexp strace_args
@@ -443,6 +478,9 @@ test_pure_prog_set()
 #   $NAME:	test name, used for "$NAME.in" file containing list of tests
 #		for positive trace expression match;
 #   $srcdir:	used to find pure_executables.list and "$NAME.in" files.
+#   $TRACE_TESTS_SAMPLE:
+#               number of binaries to pick from the remainder
+#               of pure_executables.list to check for the absence of output.
 # Files created:
 #   negative.list: File containing list of tests for negative match.
 test_trace_expr()
@@ -451,7 +489,7 @@ test_trace_expr()
 	subtrahend_regexp="$1"; shift
 	test_pure_prog_set "$@" < "$srcdir/$NAME.in"
 	prog_set_subtract "$srcdir/pure_executables.list" "$srcdir/$NAME.in" \
-		"$subtrahend_regexp" > negative.list
+		"$subtrahend_regexp" | shuf -n "${TRACE_TESTS_SAMPLE}"  > negative.list
 	test_pure_prog_set --expfile /dev/null -qq -esignal=none "$@" \
 		< negative.list
 }
@@ -527,7 +565,6 @@ test_pidns()
 
 check_scno_tampering()
 {
-	uname_r="$(uname -r)"
 	case "$STRACE_ARCH" in
 		arm)
 			# PTRACE_SET_SYSCALL is supported by linux kernel
@@ -575,9 +612,10 @@ esac
 STRACE_EXE=
 if [ -n "$NAME" ]; then
 	TESTDIR="$NAME.dir"
-	rm -rf -- "$TESTDIR"
-	mkdir -- "$TESTDIR"
-	cd "$TESTDIR"
+	rm -rf -- "$TESTDIR" &&
+	mkdir -- "$TESTDIR" &&
+	cd "$TESTDIR" ||
+	framework_failure_ "Cannot setup $TESTDIR"
 
 	case "$srcdir" in
 		/*) ;;
@@ -606,6 +644,7 @@ export STRACE_EXE
 
 : "${TIMEOUT_DURATION:=1500}"
 : "${SLEEP_A_BIT:=sleep 1}"
+: "${TRACE_TESTS_SAMPLE:=100}"
 
 [ -z "${VERBOSE-}" ] ||
 	set -x
