@@ -2,7 +2,7 @@
  * Check decoding of pipe syscall.
  *
  * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2015-2021 The strace developers.
+ * Copyright (c) 2015-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -22,7 +22,7 @@ main(void)
 {
 	(void) close(0);
 	(void) close(1);
-	int *const fds = tail_alloc(sizeof(*fds) * 2);
+	TAIL_ALLOC_OBJECT_CONST_ARR(int, fds, 2);
 	if (pipe(fds))
 		perror_msg_and_fail("pipe");
 
